@@ -26,12 +26,10 @@
 
 from __future__ import absolute_import
 
-import sys
 import inspect
 
 import six
 
-from twisted.python import log
 from twisted.internet.defer import inlineCallbacks
 
 from autobahn.wamp import protocol
@@ -39,6 +37,9 @@ from autobahn.wamp.types import ComponentConfig
 from autobahn.websocket.protocol import parseWsUrl
 from autobahn.twisted.websocket import WampWebSocketClientFactory
 
+# new API
+# from autobahn.twisted.connection import Connection
+
 import txaio
 txaio.use_twisted()
 
@@ -48,7 +49,10 @@ __all__ = [
     'ApplicationSessionFactory',
     'ApplicationRunner',
     'Application',
-    'Service'
+    'Service',
+
+    # new API
+    'Session'
 ]
 
 try:
@@ -64,16 +68,6 @@ class ApplicationSession(protocol.ApplicationSession):
     WAMP application session for Twisted-based applications.
     """
 
-    def onUserError(self, e, msg):
-        """
-        Override of wamp.ApplicationSession
-        """
-        # see docs; will print currently-active exception to the logs,
-        # which is just what we want.
-        log.err(e)
-        # also log the framework-provided error-message
-        log.err(msg)
-
 
 class ApplicationSessionFactory(protocol.ApplicationSessionFactory):
     """
@@ -82,8 +76,8 @@ class ApplicationSessionFactory(protocol.ApplicationSessionFactory):
 
     session = ApplicationSession
     """
-   The application session class this application session factory will use. Defaults to :class:`autobahn.twisted.wamp.ApplicationSession`.
-   """
+    The application session class this application session factory will use. Defaults to :class:`autobahn.twisted.wamp.ApplicationSession`.
+    """
 
 
 class ApplicationRunner(object):
@@ -95,9 +89,11 @@ class ApplicationRunner(object):
     connecting to a WAMP router.
     """
 
+    log = txaio.make_logger()
+
     def __init__(self, url, realm, extra=None, serializers=None,
                  debug=False, debug_wamp=False, debug_app=False,
-                 ssl=None):
+                 ssl=None, proxy=None):
         """
 
         :param url: The WebSocket URL of the WAMP router to connect to (e.g. `ws://somehost.com:8090/somepath`)
@@ -130,10 +126,14 @@ class ApplicationRunner(object):
             :meth:`twisted.internet.ssl.platformTrust` which tries to use
             your distribution's CA certificates.
         :type ssl: :class:`twisted.internet.ssl.CertificateOptions`
+
+        :param proxy: Explicit proxy server to use; a dict with ``host`` and ``port`` keys
+        :type proxy: dict or None
         """
         assert(type(url) == six.text_type)
         assert(type(realm) == six.text_type)
         assert(extra is None or type(extra) == dict)
+        assert(proxy is None or type(proxy) == dict)
         self.url = url
         self.realm = realm
         self.extra = extra or dict()
@@ -142,6 +142,7 @@ class ApplicationRunner(object):
         self.debug_wamp = debug_wamp
         self.debug_app = debug_app
         self.ssl = ssl
+        self.proxy = proxy
 
     def run(self, make, start_reactor=True):
         """
@@ -163,16 +164,21 @@ class ApplicationRunner(object):
             an IProtocol instance, which will actually be an instance
             of :class:`WampWebSocketClientProtocol`
         """
-        from twisted.internet import reactor
-        txaio.use_twisted()
-        txaio.config.loop = reactor
+        if start_reactor:
+            # only select framework, set loop and start logging when we are asked
+            # start the reactor - otherwise we are running in a program that likely
+            # already tool care of all this.
+            from twisted.internet import reactor
+            txaio.use_twisted()
+            txaio.config.loop = reactor
+
+            if self.debug or self.debug_wamp or self.debug_app:
+                txaio.start_logging(level='debug')
+            else:
+                txaio.start_logging(level='info')
 
         isSecure, host, port, resource, path, params = parseWsUrl(self.url)
 
-        # start logging to console
-        if self.debug or self.debug_wamp or self.debug_app:
-            log.startLogging(sys.stdout)
-
         # factory for use ApplicationSession
         def create():
             cfg = ComponentConfig(self.realm, self.extra)
@@ -181,7 +187,7 @@ class ApplicationRunner(object):
             except Exception as e:
                 if start_reactor:
                     # the app component could not be created .. fatal
-                    log.err(str(e))
+                    self.log.error(str(e))
                     reactor.stop()
                 else:
                     # if we didn't start the reactor, it's up to the
@@ -193,7 +199,7 @@ class ApplicationRunner(object):
 
         # create a WAMP-over-WebSocket transport client factory
         transport_factory = WampWebSocketClientFactory(create, url=self.url, serializers=self.serializers,
-                                                       debug=self.debug, debug_wamp=self.debug_wamp)
+                                                       proxy=self.proxy, debug=self.debug, debug_wamp=self.debug_wamp)
 
         # if user passed ssl= but isn't using isSecure, we'll never
         # use the ssl argument which makes no sense.
@@ -209,7 +215,12 @@ class ApplicationRunner(object):
             from twisted.internet.ssl import optionsForClientTLS
             context_factory = optionsForClientTLS(host)
 
-        if isSecure:
+        from twisted.internet import reactor
+        if self.proxy is not None:
+            from twisted.internet.endpoints import TCP4ClientEndpoint
+            client = TCP4ClientEndpoint(reactor, self.proxy['host'], self.proxy['port'])
+            transport_factory.contextFactory = context_factory
+        elif isSecure:
             from twisted.internet.endpoints import SSL4ClientEndpoint
             assert context_factory is not None
             client = SSL4ClientEndpoint(reactor, host, port, context_factory)
@@ -325,6 +336,8 @@ class Application(object):
     creating, debugging and running WAMP application components.
     """
 
+    log = txaio.make_logger()
+
     def __init__(self, prefix=None):
         """
 
@@ -529,7 +542,7 @@ class Application(object):
                 yield handler(*args, **kwargs)
             except Exception as e:
                 # FIXME
-                log.msg("Warning: exception in signal handler swallowed", e)
+                self.log.info("Warning: exception in signal handler swallowed", e)
 
 
 if service:
@@ -549,23 +562,33 @@ if service:
         """
         factory = WampWebSocketClientFactory
 
-        def __init__(self, url, realm, make, extra=None,
+        def __init__(self, url, realm, make, extra=None, context_factory=None,
                      debug=False, debug_wamp=False, debug_app=False):
             """
 
             :param url: The WebSocket URL of the WAMP router to connect to (e.g. `ws://somehost.com:8090/somepath`)
             :type url: unicode
+
             :param realm: The WAMP realm to join the application session to.
             :type realm: unicode
+
             :param make: A factory that produces instances of :class:`autobahn.asyncio.wamp.ApplicationSession`
                when called with an instance of :class:`autobahn.wamp.types.ComponentConfig`.
             :type make: callable
+
             :param extra: Optional extra configuration to forward to the application component.
             :type extra: dict
+
+            :param context_factory: optional, only for secure connections. Passed as contextFactory to
+                the ``listenSSL()`` call; see https://twistedmatrix.com/documents/current/api/twisted.internet.interfaces.IReactorSSL.connectSSL.html
+            :type context_factory: twisted.internet.ssl.ClientContextFactory or None
+
             :param debug: Turn on low-level debugging.
             :type debug: bool
+
             :param debug_wamp: Turn on WAMP-level debugging.
             :type debug_wamp: bool
+
             :param debug_app: Turn on app-level debugging.
             :type debug_app: bool
 
@@ -579,6 +602,7 @@ if service:
             self.debug_wamp = debug_wamp
             self.debug_app = debug_app
             self.make = make
+            self.context_factory = context_factory
             service.MultiService.__init__(self)
             self.setupService()
 
@@ -586,7 +610,7 @@ if service:
             """
             Setup the application component.
             """
-            isSecure, host, port, resource, path, params = parseWsUrl(self.url)
+            is_secure, host, port, resource, path, params = parseWsUrl(self.url)
 
             # factory for use ApplicationSession
             def create():
@@ -601,12 +625,39 @@ if service:
 
             # setup the client from a Twisted endpoint
 
-            if isSecure:
+            if is_secure:
                 from twisted.application.internet import SSLClient
-                clientClass = SSLClient
+                ctx = self.context_factory
+                if ctx is None:
+                    from twisted.internet.ssl import optionsForClientTLS
+                    ctx = optionsForClientTLS(host)
+                client = SSLClient(host, port, transport_factory, contextFactory=ctx)
             else:
+                if self.context_factory is not None:
+                    raise Exception("context_factory specified on non-secure URI")
                 from twisted.application.internet import TCPClient
-                clientClass = TCPClient
+                client = TCPClient(host, port, transport_factory)
 
-            client = clientClass(host, port, transport_factory)
             client.setServiceParent(self)
+
+
+# new API
+class Session(ApplicationSession):
+
+    def onJoin(self, details):
+        return self.on_join(details)
+
+    def onLeave(self, details):
+        return self.on_leave(details)
+
+    def onDisconnect(self):
+        return self.on_disconnect()
+
+    def on_join(self):
+        pass
+
+    def on_leave(self, details):
+        self.disconnect()
+
+    def on_disconnect(self):
+        pass
