@@ -27,6 +27,15 @@
 import abc
 import six
 
+__all__ = (
+    'IObjectSerializer',
+    'ISerializer',
+    'ITransport',
+    'ITransportHandler',
+    'ISession',
+    'IApplicationSession',
+)
+
 
 @six.add_metaclass(abc.ABCMeta)
 class IObjectSerializer(object):
@@ -66,78 +75,6 @@ class IObjectSerializer(object):
         """
 
 
-@six.add_metaclass(abc.ABCMeta)
-class IMessage(object):
-    """
-    A WAMP message.
-    """
-
-    @abc.abstractproperty
-    def MESSAGE_TYPE(self):
-        """
-        WAMP message type code.
-        """
-
-    @abc.abstractmethod
-    def marshal(self):
-        """
-        Marshal this object into a raw message for subsequent serialization to bytes.
-
-        :returns: list -- The serialized raw message.
-        """
-
-    # @abc.abstractstaticmethod ## FIXME: this is Python 3 only
-    # noinspection PyMethodParameters
-    def parse(wmsg):
-        """
-        Factory method that parses a unserialized raw message (as returned byte
-        :func:`autobahn.interfaces.ISerializer.unserialize`) into an instance
-        of this class.
-
-        :returns: obj -- An instance of this class.
-        """
-
-    @abc.abstractmethod
-    def serialize(self, serializer):
-        """
-        Serialize this object into a wire level bytes representation and cache
-        the resulting bytes. If the cache already contains an entry for the given
-        serializer, return the cached representation directly.
-
-        :param serializer: The wire level serializer to use.
-        :type serializer: An instance that implements :class:`autobahn.interfaces.ISerializer`
-
-        :returns: bytes -- The serialized bytes.
-        """
-
-    @abc.abstractmethod
-    def uncache(self):
-        """
-        Resets the serialization cache.
-        """
-
-    @abc.abstractmethod
-    def __eq__(self, other):
-        """
-        Message equality. This does an attribute-wise comparison (but skips attributes
-        that start with `_`).
-        """
-
-    @abc.abstractmethod
-    def __ne__(self, other):
-        """
-        Message inequality (just the negate of message equality).
-        """
-
-    @abc.abstractmethod
-    def __str__(self):
-        """
-        Returns text representation of this message.
-
-        :returns: str -- Human readable representation (e.g. for logging or debugging purposes).
-        """
-
-
 @six.add_metaclass(abc.ABCMeta)
 class ISerializer(object):
     """
@@ -159,23 +96,26 @@ class ISerializer(object):
     @abc.abstractmethod
     def serialize(self, message):
         """
-        Serializes a WAMP message to bytes to be sent to a transport.
+        Serializes a WAMP message to bytes for sending over a transport.
 
         :param message: An instance that implements :class:`autobahn.wamp.interfaces.IMessage`
         :type message: obj
 
-        :returns: tuple -- A pair ``(bytes, isBinary)``.
+        :returns: tuple -- A pair ``(payload, is_binary)``.
         """
 
     @abc.abstractmethod
-    def unserialize(self, payload, isBinary):
+    def unserialize(self, payload, is_binary):
         """
         Deserialize bytes from a transport and parse into WAMP messages.
 
         :param payload: Byte string from wire.
         :type payload: bytes
+        :param is_binary: Type of payload. True if payload is a binary string, else
+            the payload is UTF-8 encoded Unicode text.
+        :type is_binary: bool
 
-        :returns: list -- List of objects that implement :class:`autobahn.wamp.interfaces.IMessage`.
+        :returns: list -- List of ``a.w.m.Message`` objects.
         """
 
 
@@ -191,13 +131,18 @@ class ITransport(object):
         """
         Send a WAMP message over the transport to the peer. If the transport is
         not open, this raises :class:`autobahn.wamp.exception.TransportLost`.
+        Returns a deferred/future when the message has been processed and more
+        messages may be sent. When send() is called while a previous deferred/future
+        has not yet fired, the send will fail immediately.
 
         :param message: An instance that implements :class:`autobahn.wamp.interfaces.IMessage`
         :type message: obj
+
+        :returns: obj -- A Deferred/Future
         """
 
     @abc.abstractmethod
-    def isOpen(self):
+    def is_open(self):
         """
         Check if the transport is open for messaging.
 
@@ -225,31 +170,43 @@ class ITransport(object):
 @six.add_metaclass(abc.ABCMeta)
 class ITransportHandler(object):
 
-    @abc.abstractmethod
-    def onOpen(self, transport):
+    @abc.abstractproperty
+    def transport(self):
         """
-        Callback fired when transport is open.
+        When the transport this handler is attached to is currently open, this property
+        can be read from. The property should be considered read-only. When the transport
+        is gone, this property is set to None.
+        """
+
+    @abc.abstractmethod
+    def on_open(self, transport):
+        """
+        Callback fired when transport is open. May run asynchronously. The transport
+        is considered running and is_open() would return true, as soon as this callback
+        has completed successfully.
 
         :param transport: An instance that implements :class:`autobahn.wamp.interfaces.ITransport`
         :type transport: obj
         """
 
     @abc.abstractmethod
-    def onMessage(self, message):
+    def on_message(self, message):
         """
-        Callback fired when a WAMP message was received.
+        Callback fired when a WAMP message was received. May run asynchronously. The callback
+        should return or fire the returned deferred/future when it's done processing the message.
+        In particular, an implementation of this callback must not access the message afterwards.
 
         :param message: An instance that implements :class:`autobahn.wamp.interfaces.IMessage`
         :type message: obj
         """
 
     @abc.abstractmethod
-    def onClose(self, wasClean):
+    def on_close(self, was_clean):
         """
         Callback fired when the transport has been closed.
 
-        :param wasClean: Indicates if the transport has been closed regularly.
-        :type wasClean: bool
+        :param was_clean: Indicates if the transport has been closed regularly.
+        :type was_clean: bool
         """
 
 
@@ -260,7 +217,7 @@ class ISession(object):
     """
 
     @abc.abstractmethod
-    def onConnect(self):
+    def on_connect(self):
         """
         Callback fired when the transport this session will run over has been established.
         """
@@ -272,7 +229,7 @@ class ISession(object):
         """
 
     @abc.abstractmethod
-    def onChallenge(self, challenge):
+    def on_challenge(self, challenge):
         """
         Callback fired when the peer demands authentication.
 
@@ -283,7 +240,7 @@ class ISession(object):
         """
 
     @abc.abstractmethod
-    def onJoin(self, details):
+    def on_join(self, details):
         """
         Callback fired when WAMP session has been established.
 
@@ -308,7 +265,7 @@ class ISession(object):
         """
 
     @abc.abstractmethod
-    def onLeave(self, details):
+    def on_leave(self, details):
         """
         Callback fired when WAMP session has is closed
 
@@ -329,11 +286,19 @@ class ISession(object):
         """
 
     @abc.abstractmethod
-    def onDisconnect(self):
+    def on_disconnect(self):
         """
         Callback fired when underlying transport has been closed.
         """
 
+
+@six.add_metaclass(abc.ABCMeta)
+class IApplicationSession(ISession):
+    """
+    Interface for WAMP client peers implementing the four different
+    WAMP roles (caller, callee, publisher, subscriber).
+    """
+
     @abc.abstractmethod
     def define(self, exception, error=None):
         """
@@ -346,12 +311,6 @@ class ISession(object):
         :type error: str
         """
 
-
-class ICaller(ISession):
-    """
-    Interface for WAMP peers implementing role *Caller*.
-    """
-
     @abc.abstractmethod
     def call(self, procedure, *args, **kwargs):
         """
@@ -386,54 +345,6 @@ class ICaller(ISession):
         :rtype: instance of :tx:`twisted.internet.defer.Deferred` / :py:class:`asyncio.Future`
         """
 
-
-@six.add_metaclass(abc.ABCMeta)
-class IRegistration(object):
-    """
-    Represents a registration of an endpoint.
-    """
-
-    @abc.abstractproperty
-    def id(self):
-        """
-        The WAMP registration ID for this registration.
-        """
-
-    @abc.abstractproperty
-    def active(self):
-        """
-        Flag indicating if registration is active.
-        """
-
-    @abc.abstractmethod
-    def unregister(self):
-        """
-        Unregister this registration that was previously created from
-        :func:`autobahn.wamp.interfaces.ICallee.register`.
-
-        After a registration has been unregistered successfully, no calls
-        will be routed to the endpoint anymore.
-
-        Returns an instance of :tx:`twisted.internet.defer.Deferred` (when
-        running on **Twisted**) or an instance of :py:class:`asyncio.Future`
-        (when running on **asyncio**).
-
-        - If the unregistration succeeds, the returned Deferred/Future will
-          *resolve* (with no return value).
-
-        - If the unregistration fails, the returned Deferred/Future will be rejected
-          with an instance of :class:`autobahn.wamp.exception.ApplicationError`.
-
-        :returns: A Deferred/Future for the unregistration
-        :rtype: instance(s) of :tx:`twisted.internet.defer.Deferred` / :py:class:`asyncio.Future`
-        """
-
-
-class ICallee(ISession):
-    """
-    Interface for WAMP peers implementing role *Callee*.
-    """
-
     @abc.abstractmethod
     def register(self, endpoint, procedure=None, options=None):
         """
@@ -467,25 +378,6 @@ class ICallee(ISession):
         :rtype: instance(s) of :tx:`twisted.internet.defer.Deferred` / :py:class:`asyncio.Future`
         """
 
-
-@six.add_metaclass(abc.ABCMeta)
-class IPublication(object):
-    """
-    Represents a publication of an event. This is used with acknowledged publications.
-    """
-
-    @abc.abstractproperty
-    def id(self):
-        """
-        The WAMP publication ID for this publication.
-        """
-
-
-class IPublisher(ISession):
-    """
-    Interface for WAMP peers implementing role *Publisher*.
-    """
-
     @abc.abstractmethod
     def publish(self, topic, *args, **kwargs):
         """
@@ -520,54 +412,6 @@ class IPublisher(ISession):
         :rtype: ``None`` or instance of :tx:`twisted.internet.defer.Deferred` / :py:class:`asyncio.Future`
         """
 
-
-@six.add_metaclass(abc.ABCMeta)
-class ISubscription(object):
-    """
-    Represents a subscription to a topic.
-    """
-
-    @abc.abstractproperty
-    def id(self):
-        """
-        The WAMP subscription ID for this subscription.
-        """
-
-    @abc.abstractproperty
-    def active(self):
-        """
-        Flag indicating if subscription is active.
-        """
-
-    @abc.abstractmethod
-    def unsubscribe(self):
-        """
-        Unsubscribe this subscription that was previously created from
-        :func:`autobahn.wamp.interfaces.ISubscriber.subscribe`.
-
-        After a subscription has been unsubscribed successfully, no events
-        will be routed to the event handler anymore.
-
-        Returns an instance of :tx:`twisted.internet.defer.Deferred` (when
-        running on **Twisted**) or an instance of :py:class:`asyncio.Future`
-        (when running on **asyncio**).
-
-        - If the unsubscription succeeds, the returned Deferred/Future will
-          *resolve* (with no return value).
-
-        - If the unsubscription fails, the returned Deferred/Future will *reject*
-          with an instance of :class:`autobahn.wamp.exception.ApplicationError`.
-
-        :returns: A Deferred/Future for the unsubscription
-        :rtype: instance(s) of :tx:`twisted.internet.defer.Deferred` / :py:class:`asyncio.Future`
-        """
-
-
-class ISubscriber(ISession):
-    """
-    Interface for WAMP peers implementing role *Subscriber*.
-    """
-
     @abc.abstractmethod
     def subscribe(self, handler, topic=None, options=None):
         """
